@@ -13,10 +13,38 @@
 package require tdbc
 package require sqlite3
 
-package provide tdbc::sqlite3 1.1.1
+package provide tdbc::sqlite3 1.1.8
 
 namespace eval tdbc::sqlite3 {
     namespace export connection
+    variable script [info script]
+
+    proc build-info args {
+	if {[llength $args] > 0} {
+	    if {[llength $args] > 1} {
+		return -code error "wrong # args: should be \"[lrange [info level 0] 0 end-[llength $args]]\""
+	    }
+	    switch -exact -- [lindex $args 0] {
+		commit {}
+		compiler {return none}
+		patchlevel {return [package provide tdbc::sqlite3]}
+		version {return [join [lrange [split [package provide tdbc::sqlite3] .] 0 1] .]}
+		default {return 0}
+	    }
+	}
+	variable script
+	set id unknown
+	catch {
+	    set f [open [file rootname $script].uuid]
+	    set id [string trim [read $f]]
+	    close $f
+	}
+	if {[llength $args] > 0} {
+	    return $id
+	} else {
+	    return [package provide tdbc::sqlite3]+$id
+	}
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -32,11 +60,13 @@ namespace eval tdbc::sqlite3 {
     superclass ::tdbc::connection
 
     variable timeout
+    variable keepcase
 
     # The constructor accepts a database name and opens the database.
 
     constructor {databaseName args} {
 	set timeout 0
+	set keepcase 0
 	if {[llength $args] % 2 != 0} {
 	    set cmd [lrange [info level 0] 0 end-[llength $args]]
 	    return -code error \
@@ -70,6 +100,7 @@ namespace eval tdbc::sqlite3 {
 	    } else {
 		lappend result serializable
 	    }
+	    lappend result -keepcase $keepcase
 	    lappend result -readonly 0
 	    lappend result -timeout $timeout
 	    return $result
@@ -79,34 +110,33 @@ namespace eval tdbc::sqlite3 {
 	    # Query a single option
 
 	    set option [lindex $args 0]
-	    switch -exact -- $option {
-		-e - -en - -enc - -enco - -encod - -encodi - -encodin -
-		-encoding {
+	    if {[catch {::tcl::prefix match -message "option" {
+		-encoding -isolation -keepcase -readonly -timeout
+	    } $option} opt]} {
+		return -code error \
+			-errorcode [list TDBC GENERAL_ERROR HY000 SQLITE3 \
+					BADOPTION $option] $opt
+	    }
+	    switch -exact -- $opt {
+ 		-encoding {
 		    return utf-8
 		}
-		-i - -is - -iso - -isol - -isola - -isolat - -isolati -
-		-isolatio - -isolation {
+		-isolation {
 		    if {[db onecolumn {PRAGMA read_uncommitted}]} {
 			return readuncommitted
 		    } else {
 			return serializable
 		    }
 		}
-		-r - -re - -rea - -read - -reado - -readon - -readonl -
+		-keepcase {
+		    return $keepcase
+		}
 		-readonly {
 		    return 0
 		}
-		-t - -ti - -tim - -time - -timeo - -timeou - -timeout {
+		-timeout {
 		    return $timeout
 		}
-		default {
-		    return -code error \
-			-errorcode [list TDBC GENERAL_ERROR HY000 SQLITE3 \
-					BADOPTION $option] \
-			"bad option \"$option\": must be\
-                         -encoding, -isolation, -readonly or -timeout"
-
-		}
 	    }
 
 	} elseif {[llength $args] % 2 != 0} {
@@ -123,48 +153,56 @@ namespace eval tdbc::sqlite3 {
 	# Set one or more options
 
 	foreach {option value} $args {
-	    switch -exact -- $option {
-		-e - -en - -enc - -enco - -encod - -encodi - -encodin -
+	    if {[catch {::tcl::prefix match -message "option" {
+		-encoding -isolation -keepcase -readonly -timeout
+	    } $option} opt]} {
+		return -code error \
+			-errorcode [list TDBC GENERAL_ERROR HY000 SQLITE3 \
+					BADOPTION $option] $opt
+	    }
+
+	    switch -exact -- $opt {
 		-encoding {
 		    if {$value ne {utf-8}} {
 			return -code error \
 			    -errorcode [list TDBC FEATURE_NOT_SUPPORTED 0A000 \
 					    SQLITE3 ENCODING] \
-			    "-encoding not supported. SQLite3 is always \
+			    "-encoding not supported. SQLite3 is always\
                              Unicode."
 		    }
 		}
-		-i - -is - -iso - -isol - -isola - -isolat - -isolati -
-		-isolatio - -isolation {
-		    switch -exact -- $value {
-			readu - readun - readunc - readunco - readuncom -
-			readuncomm - readuncommi - readuncommit -
-			readuncommitt - readuncommitte - readuncommitted {
+		-isolation {
+		    if {[catch {::tcl::prefix match -message "isolation level" {
+			readuncommitted readcommitted repeatableread
+			serializable readonly
+		    } $value} val]} {
+			return -code error \
+			    -errorcode [list TDBC GENERAL_ERROR HY000 \
+					    SQLITE3 BADISOLATION $value] \
+			    $val
+		    }
+		    switch -exact -- $val {
+			readuncommitted {
 			    db eval {PRAGMA read_uncommitted = 1}
 			}
-			readc - readco - readcom - readcomm - readcommi -
-			readcommit - readcommitt - readcommitte -
 			readcommitted -
-			rep - repe - repea - repeat - repeata - repeatab -
-			repeatabl - repeatable - repeatabler - repeatablere -
-			repeatablerea - repeatablread -
-			s - se - ser - seri - seria - serial - seriali -
-			serializ - serializa - serializab - serializabl -
+			repeatableread -
 			serializable -
-			reado - readon - readonl - readonly {
+			readonly {
 			    db eval {PRAGMA read_uncommitted = 0}
 			}
-			default {
-			    return -code error \
-				-errorcode [list TDBC GENERAL_ERROR HY000 \
-						SQLITE3 BADISOLATION $value] \
-				"bad isolation level \"$value\":\
-                                should be readuncommitted, readcommitted,\
-                                repeatableread, serializable, or readonly"
-			}
 		    }
 		}
-		-r - -re - -rea - -read - -reado - -readon - -readonl -
+		-keepcase {
+		    if {![string is boolean -strict $value]} {
+			return -code error \
+			    -errorcode [list TDBC DATA_EXCEPTION 22018 \
+					    SQLITE3 $value] \
+			    "expected boolean but got \"$value\""
+		    }
+		    # Normalize boolean value to 0/1
+		    set keepcase [expr {bool($value)}]
+		}
 		-readonly {
 		    if {$value} {
 			return -code error \
@@ -174,8 +212,8 @@ namespace eval tdbc::sqlite3 {
                              access"
 		    }
 		}
-		-t - -ti - -tim - -time - -timeo - -timeou - -timeout {
-		    if {![string is integer $value]} {
+		-timeout {
+		    if {![string is integer -strict $value]} {
 			return -code error \
 			    -errorcode [list TDBC DATA_EXCEPTION 22018 \
 					    SQLITE3 $value] \
@@ -184,14 +222,6 @@ namespace eval tdbc::sqlite3 {
 		    db timeout $value
 		    set timeout $value
 		}
-		default {
-		    return -code error \
-			-errorcode [list TDBC GENERAL_ERROR HY000 \
-					SQLITE3 BADOPTION $value] \
-			"bad option \"$option\": must be\
-                         -encoding, -isolation, -readonly or -timeout"
-
-		}
 	    }
 	}
 	return
@@ -206,7 +236,9 @@ namespace eval tdbc::sqlite3 {
 	    WHERE type IN ('table', 'view')
 	    AND name LIKE :pattern
 	} {
-	    dict set row name [string tolower [dict get $row name]]
+	    if {!$keepcase} {
+		dict set row name [string tolower [dict get $row name]]
+	    }
 	    dict set retval [dict get $row name] $row
 	}
 	return $retval
@@ -229,13 +261,17 @@ namespace eval tdbc::sqlite3 {
 	    dict for {key value} $origrow {
 		dict set row [string tolower $key] $value
 	    }
-	    dict set row name [string tolower [dict get $row name]]
-	    if {![string match $pattern [dict get $row name]]} {
+	    if {!$keepcase} {
+		dict set row name [string tolower [dict get $row name]]
+	    }
+	    if {![string match $pattern \
+		      [string tolower [dict get $row name]]]} {
 		continue
 	    }
 	    switch -regexp -matchvar info [dict get $row type] {
 		{^(.+)\(\s*([[:digit:]]+)\s*,\s*([[:digit:]]+)\s*\)\s*$} {
-		    dict set row type [string tolower [lindex $info 1]]
+		    dict set row type \
+			    [string trim [string tolower [lindex $info 1]]]
 		    dict set row precision [lindex $info 2]
 		    dict set row scale [lindex $info 3]
 		}
@@ -354,7 +390,7 @@ namespace eval tdbc::sqlite3 {
 	my foreach row "PRAGMA foreign_key_list($foreignTable)" {
 	    if {(![dict exists $argdict primary])
 		|| ([string tolower [dict get $row table]]
-		    eq [dict get $argdict primary])} {
+		    eq [string tolower [dict get $argdict primary]])} {
 
 		# Construct a dictionary for each key, translating
 		# SQLite names to TDBC ones and converting sequence
